package asm

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/register"
)

type InstructionList struct {
	Instructions []Instruction
}

// Finalize generates the final assembly code.
func (list *InstructionList) Finalize() *Result {
	final := Result{}

	for _, instr := range list.Instructions {
		instr.Write(&final.Code)
		fmt.Println(instr.String())
	}

	return &final
}

func (list *InstructionList) MoveRegisterNumber(reg register.Register, number uint64) {
	list.addRegisterNumber(MOV, reg, number)
}

func (list *InstructionList) Syscall() {
	list.add(SYSCALL)
}

// add adds an instruction without any operands.
func (list *InstructionList) add(mnemonic Mnemonic) {
	list.Instructions = append(list.Instructions, &Base{Mnemonic: mnemonic})
}

// addRegisterNumber adds an instruction using a register and a number.
func (list *InstructionList) addRegisterNumber(mnemonic Mnemonic, reg register.Register, number uint64) {
	list.Instructions = append(list.Instructions, &RegisterNumber{
		Base: Base{
			Mnemonic: mnemonic,
		},
		Register: reg,
		Number:   number,
	})
}
